/* $Id: $ */

#ifndef _IMAGE_IMAGE_PREPROCESSOR_HPP_
#define _IMAGE_IMAGE_PREPROCESSOR_HPP_

#include <list>
#include <image/Image.hpp>

namespace jafar {
  namespace image {
    /**
     * Virtual interface to implement to add a different preprocess type for images.
     */
    class ImagePreprocessorNode {
      public:
        /**
         * Preprocess an image. In most case the size of src and dst must be equal. And
         * it is possible to use dst == src
         * @param src
         * @param dst
         */
        virtual void preprocessImage(const jafar::image::Image& src, jafar::image::Image& dst) =0;
        virtual bool hasWork() { return true; }

        virtual ~ImagePreprocessorNode() { }
      
    };
    /**
     * This node contains a list of image preprocessor.
     */
    class ImagePreprocessor : public ImagePreprocessorNode {
      public:
        /**
         * Initialize an image preprocessor with an empty list of nodes. If the size of the
         * buffer image is the size of the src images that will be processed, nodes will
         * be applied first and resizing last. If the size of the buffer image is the size
         * of the dst images, resizing will be done first and nodes applied afterwards.
         *
         * @param width the width of the buffer image (can be either equal to src or dst)
         * @param width the height of the buffer image (can be either equal to src or dst)
         * @param depth the depth of the buffer image
         * @param colorSpace the colorSpace of the buffer image (must be equal to the dst image)
         */
        ImagePreprocessor(int width, int height, int depth, JfrImage_TypeColorSpace colorSpace);
        virtual ~ImagePreprocessor();
        void appendNode(ImagePreprocessorNode* node)
          { m_nodes.push_back(node); m_nodeCount++; }
        /**
         * Will use temporary buffers if needed.
         */
        virtual void preprocessImage(const jafar::image::Image& src, jafar::image::Image& dst);
        virtual void preprocessImage2(const jafar::image::Image& src, jafar::image::Image& dst);
      private:
        jafar::image::Image* m_buffer;
        jafar::image::Image* m_buffer2;
        jafar::image::Image* m_buffer3;
        std::list<ImagePreprocessorNode*> m_nodes;
        int m_nodeCount;
    };
  }
}



#endif
