/* $Id$ */

#ifndef _JAFAR_IMAGE_CIMAGE_H_
#define _JAFAR_IMAGE_CIMAGE_H_
#include <opencv/cxcore.h>
#if CV_MAJOR_VERSION*10 + CV_MINOR_VERSION > 21
#include <opencv2/core/core_c.h>
#include <opencv2/core/types_c.h>
#include <opencv2/imgproc/types_c.h>
#include <opencv2/imgproc/imgproc_c.h>
#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#else
#include <cv.h> // opencv 
#include <cxtypes.h> // opencv 
#include <cxcore.h> // opencv
#endif
/** \addtogroup image */
/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

  /** Available colorspaces for JfrImage.  Although OpenCV functions only handle
   * @c JfrImage_CS_GRAY and @c JfrImage_CS_BGR images, it is possible to define
   * other colorspace.  
   *
   * @note If another colorspace is needed, you may add it.
   */
  typedef enum JfrImage_TypeColorSpace {
    JfrImage_CS_GRAY,
    JfrImage_CS_BGR,
    JfrImage_CS_RGB,
    JfrImage_CS_HSV,
    JfrImage_CS_HSL,
    JfrImage_CS_CMY,
    JfrImage_CS_CMYK,
    JfrImage_CS_LUV,
    JfrImage_CS_LAB,
    JfrImage_CS_XYZ,
    JfrImage_CS_YXY,
    JfrImage_CS_BayerBG,
    JfrImage_CS_BayerGB,
    JfrImage_CS_BayerRG,
    JfrImage_CS_BayerGR,
    JfrImage_CS_YCbCr,
    JfrImage_CS_UNKNOWN} JfrImage_TypeColorSpace;

  typedef enum JfrImage_InterpolationType {
    JfrImage_INTERP_NEAREST,
    JfrImage_INTERP_BILINEAR,
    JfrImage_INTERP_CUBIC
  } JfrImage_InterpolationType;
  /** Returns the number of channels required by the color space \a cs.
   * Return 0 for JfrImage_CS_UNKNOWN.
   */
  int JfrImage_colorSpaceChannels(JfrImage_TypeColorSpace cs);

  /** Returns the size in bytes of one channel of a pixel */
  int channelDepthSize(int depth);

  /** Jafar supplementary header. It is defined in C for compatibility (GenoM).
   */
  struct JfrImage_ImageHeader {
    JfrImage_TypeColorSpace colorSpace;
  };

  /** Jafar image C structure. It is defined for compatibility (GenoM).
   */
  typedef struct JfrImage_CImage {
    struct JfrImage_ImageHeader header;
    IplImage iplImage;
  } JfrImage_CImage;

  /** C interface to the convertColorMode function.
   */
  void JfrImage_convertColorMode(const JfrImage_CImage* src_, JfrImage_CImage* dst_);

#ifdef __cplusplus
}
#endif

/*@}*/
/* End of Doxygen group */

#endif // _JAFAR_IMAGE_CIMAGE_H_
