// boost unit test includes
#include <boost/test/auto_unit_test.hpp>

// jafar debug include
#include "kernel/jafarDebug.hpp"
#include "kernel/jafarTestMacro.hpp"
#include <fstream>

#include <iostream>
#include "image/roi.hpp"

using namespace std;
using namespace jafar::image;




BOOST_AUTO_TEST_CASE( test_roi )
{std::cout << "TEST!!!" << std::endl;
	ConvexRoi roi1(1,2,3,4);
	JFR_CHECK_STRICT_EQUAL(roi1.x(), 1);
	JFR_CHECK_STRICT_EQUAL(roi1.y(), 2);
	JFR_CHECK_STRICT_EQUAL(roi1.w(), 3);
	JFR_CHECK_STRICT_EQUAL(roi1.h(), 4);
	cv::Rect rect2(2,3,4,5);
	ConvexRoi roi2(rect2);
	JFR_CHECK_STRICT_EQUAL(roi2.x(), 2);
	JFR_CHECK_STRICT_EQUAL(roi2.y(), 3);
	JFR_CHECK_STRICT_EQUAL(roi2.w(), 4);
	JFR_CHECK_STRICT_EQUAL(roi2.h(), 5);
	
	roi1 *= roi2;
	JFR_CHECK_STRICT_EQUAL(roi1.x(), 2);
	JFR_CHECK_STRICT_EQUAL(roi1.y(), 3);
	JFR_CHECK_STRICT_EQUAL(roi1.w(), 2);
	JFR_CHECK_STRICT_EQUAL(roi1.h(), 3);
	
	jblas::vec x(2); x(0) = 20; x(1) = 20;
	jblas::sym_mat P(2,2); P(0,0)=15*15; P(1,1)=5*5; P(0,1)=37;
	ConvexRoi roi3(x, P, 1.0);
	roi3.draw();
	roi3.checkIntegrity();
	ConvexRoi roi3b(x, P, 2.0);
	roi3b.draw();
	roi3b.checkIntegrity();
	
	cv::Rect rect4(8,0,40,23);
	ConvexRoi roi4(rect4);
	ConvexRoi roi5 = roi3*roi4;
	roi5.draw();
	roi5.checkIntegrity();
	
	P(0,1)=-70;
	ConvexRoi roi3c(x, P, 1.0);
	roi3c.draw();
	roi3c.checkIntegrity();
	cv::Rect rect4b(6,18,7,5);
	ConvexRoi roi4b(rect4b);
	ConvexRoi roi5b = roi3c*roi4b;
	roi5b.draw();
	roi5b.checkIntegrity();
	
	roi4 *= roi3;
	roi4.draw();
	roi4.checkIntegrity();
	
	cv::Rect rect7(4,17,27,23);
	ConvexRoi roi7(rect7);
	roi3 *= roi7;
	roi3.draw();
	roi3.checkIntegrity();
	
	x(0) = 50; x(1) = 50;
	P(0,0)=15*15; P(1,1)=5*5; P(0,1)=37;
	ConvexRoi roi6(x, P, 2.0);
	roi6.draw();
	roi6.checkIntegrity();
	ConvexRoi roi8 = roi6 * 0.5;
	roi8.draw();
	roi8.checkIntegrity();
	cv::Rect rect10(20,40,60,22);
	ConvexRoi roi10(rect10);
	roi10 *= roi6;
	roi10.draw();
	roi10.checkIntegrity();
	roi10 *= 0.6;
	roi10.draw();
	roi10.checkIntegrity();
	
	cv::Rect roi11(0,0,640,480);
	P.clear(); P(0,0)=P(1,1)=1.0;
	x(0)=13.35; x(1)=481.24;
	ConvexRoi roi12(x, P, 1.0);
	roi12.checkIntegrity();
	ConvexRoi roi13 = roi12 * roi11;
	std::cout << roi13 << std::endl;
	roi13.checkIntegrity();

	x(0)=641.35; x(1)=81.24;
	ConvexRoi roi14(x, P, 1.0);
	roi14.checkIntegrity();
	roi13 = roi14 * roi11;
	std::cout << roi13 << std::endl;
	roi13.checkIntegrity();
	
	x(0)=640.35; x(1)=480.24;
	ConvexRoi roi15(x, P, 1.0);
	roi15.checkIntegrity();
	roi13 = roi15 * roi11;
	std::cout << roi13 << std::endl;
	roi13.checkIntegrity();
	
	x(0)=642.35; x(1)=482.24;
	ConvexRoi roi16(x, P, 1.0);
	roi16.checkIntegrity();
	roi13 = roi16 * roi11;
	std::cout << roi13 << std::endl;
	roi13.checkIntegrity();
}


